/*
 * Gnophone: A client for the Asterisk PBX
 *
 * Copyright (C) 2000, Linux Support Services, Inc.
 *
 * Written by Mark Spencer
 *
 * Linux/UNIX version distributed under the terms of
 * the GNU General Public License
 *
 * html-csc.c CSC-Html based browser back-end
 *
 */

#include <stdio.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>                                                   
#include <string.h>

#include <gtk/gtk.h>
#include <cschtml/cschtml.h>
#include <ghttp.h>

#include "gnophone.h"
#include "htmlengine.h"

struct html_pvt {
	GtkWidget *cscw;
	CscHTML *csc;
	GtkWidget *sw;
	ghttp_request *req;
	CscHTMLStream *handle;
	int redirid;
	int netid;
	char *uri;
	char *baseuri;
};


static void show_status(ghttp_current_status cs)
{
	printf("proc: %d, %d of %d bytes\n", cs.proc, cs.bytes_read, cs.bytes_total);
}

static void process_req(struct browser *b)
{
	csc_html_begin(b->pvt->csc);
	csc_html_write(b->pvt->csc, b->pvt->handle, ghttp_get_body(b->pvt->req), ghttp_get_body_len(b->pvt->req));
	csc_html_end(b->pvt->csc, b->pvt->handle, CSC_HTML_STREAM_OK);
}

static void netdata(void *vb, int fd, GdkInputCondition c)
{
	struct browser *b = vb;
	ghttp_status s;	
	ghttp_current_status cs;
	s = ghttp_process(b->pvt->req);
	cs = ghttp_get_status(b->pvt->req);
	show_status(cs);
	switch(s) {
	case ghttp_error:
		fprintf(stderr, "Error loading URL\n");
		break;
	case ghttp_done:
		fprintf(stderr, "Already done loading URL\n");
		process_req(b);
		gdk_input_remove(b->pvt->netid);
		b->pvt->netid = 0;
		break;
	case ghttp_not_done:
		fprintf(stderr, "Not done yet loading url\n");
		break;
	};
}

static void title_changed_cb(CscHTML *html, const gchar *title, struct browser *b)
{
	printf("New title: %s\n", title);
}

static void url_requested(CscHTML *html, const gchar *url, CscHTMLStream *handle, struct browser *b)
{
	ghttp_status s;
	ghttp_current_status cs;
	printf("URL requested: %s\n", url);
	if (b->pvt->req)
		ghttp_request_destroy(b->pvt->req);
	b->pvt->req = ghttp_request_new();
	ghttp_set_uri(b->pvt->req, (char *)url);
#if 0
	ghttp_set_sync(b->pvt->req, ghttp_async);
#endif
	ghttp_prepare(b->pvt->req);
	s = ghttp_process(b->pvt->req);
	cs = ghttp_get_status(b->pvt->req);
	show_status(cs);
	switch(s) {
	case ghttp_error:
		fprintf(stderr, "Error loading URL %s\n", url);
		break;
	case ghttp_done:
		fprintf(stderr, "Already done loading URL %s\n", url);
		process_req(b);
		break;
	case ghttp_not_done:
		fprintf(stderr, "Not done yet loading url %s\n", url);
		b->pvt->netid = gdk_input_add(ghttp_get_socket(b->pvt->req), GDK_INPUT_READ | GDK_INPUT_EXCEPTION, netdata, b);
		break;
	};
	
}

static void load_done(CscHTML *html, struct browser *b)
{
	printf("Done with load\n");
}

static void on_set_base(CscHTML *html, const gchar *url, struct browser *b)
{
	printf("Setting base URL to %s\n", url);
}

static void on_url(CscHTML *html, const gchar *url, struct browser *b)
{
	printf("Looking at URL %s\n", url);
}

static void on_link_clicked(CscHTML *html, const gchar *url, struct browser *b)
{
	printf("Clicked link %s\n", url);
} 

static void on_redirect(CscHTML *html, const gchar *url, int delay, struct browser *b)
{
	printf("Goto URL %s in %d seconds\n", url, delay);
}

static void on_submit(CscHTML *html, const char *method, const gchar *action, const gchar *encoding, struct browser *b)
{
	printf("Submit by %s, to %s, encoding: %s\n", method, action, encoding);
}

static void object_requested_cmd(CscHTML *html, CscHTMLEmbedded *eb, void *data)
{
	printf("What the heck?\n");
}

static void on_button_press_event(GtkWidget *widget, GdkEventButton *event, struct browser *b)
{
	printf("Click!\n");
}

static void set_signals(CscHTML *html, struct browser *b)
{
	gtk_signal_connect(GTK_OBJECT(html), "title_changed", GTK_SIGNAL_FUNC(title_changed_cb), (gpointer)b);
	gtk_signal_connect(GTK_OBJECT(html), "url_requested", GTK_SIGNAL_FUNC(url_requested), (gpointer)b);
	gtk_signal_connect(GTK_OBJECT(html), "load_done", GTK_SIGNAL_FUNC(load_done), (gpointer)b);
	gtk_signal_connect(GTK_OBJECT(html), "on_url", GTK_SIGNAL_FUNC(on_url), (gpointer)b);
	gtk_signal_connect(GTK_OBJECT(html), "set_base", GTK_SIGNAL_FUNC(on_set_base), (gpointer)b);
	gtk_signal_connect(GTK_OBJECT(html), "button_press_event", GTK_SIGNAL_FUNC(on_button_press_event), (gpointer)b);
	gtk_signal_connect(GTK_OBJECT(html), "link_clicked", GTK_SIGNAL_FUNC(on_link_clicked), (gpointer)b);
	gtk_signal_connect(GTK_OBJECT(html), "redirect", GTK_SIGNAL_FUNC(on_redirect), (gpointer)b);
	gtk_signal_connect(GTK_OBJECT(html), "submit", GTK_SIGNAL_FUNC(on_submit), (gpointer)b);
	gtk_signal_connect(GTK_OBJECT(html), "object_requested", GTK_SIGNAL_FUNC(object_requested_cmd), (gpointer)b);
}

static GtkWidget *csc_new_engine(struct browser *b)
{
	struct html_pvt *p = g_new0(struct html_pvt, 1);
	p->csc = CSC_HTML((p->cscw = csc_html_new()));
	csc_html_load_empty(p->csc);
	set_signals(p->csc, b);
	p->sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(p->sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(p->sw), p->cscw);
	b->pvt = p;
	printf("Returning new widget\n");
	return p->sw;
}

static int csc_can_go_forward(struct browser *b)
{
	return 1;
}

static int csc_can_go_back(struct browser *b)
{
	return 1;
}

static void csc_go_back(struct browser *b)
{
	printf("Backward!\n");
}

static void csc_go_forward(struct browser *b)
{
	printf("Forward!\n");
}

static void csc_reload(struct browser *b, int force)
{
	printf("Reload!\n");
}

static char *make_base(char *uri)
{
	
}

static void load_url(struct browser *b, char *uri)
{
	if (b->baseurl)
		g_free(b->baseurl);
	b->baseurl = make_base(uri);
	if (b->pvt->redirid) 
		gtk_timeout_remove(b->pvt->redirid);
	if (b->pvt->netid) 
		gtk_timeout_remove(b->pvt->netid);
	b->pvt->redirid = 0;
	b->pvt->netid = 0;
	b->pvt->handle = csc_html_begin(b->pvt->csc);
	url_requested(b->pvt->csc, uri, b->pvt->handle, b);
	
}

static void csc_loadurl(struct browser *b, char *uri)
{
	printf("Loading URL %s\n", uri);
	load_url(b, uri);
}

static void csc_stop(struct browser *b)
{
	printf("Stop!\n");
}

static struct html_engine csc;

char *key() 
{
	return KEY;
}

char *name()
{
	return "cschtml";
}

int init()
{
	strcpy(csc.name, "csc");
	csc.new_engine = csc_new_engine;
	csc.can_go_back = csc_can_go_back;
	csc.can_go_forward = csc_can_go_forward;
	csc.go_back = csc_go_back;
	csc.go_forward = csc_go_forward;
	csc.reload = csc_reload;
	csc.loadurl = csc_loadurl;
	csc.stop = csc_stop;
	csc.next = NULL;
	html_register_engine(&csc);
	return 0;
}

